package amq.test;

import javax.jms.Connection;
import javax.jms.ConnectionFactory;
import javax.jms.JMSException;
import javax.jms.Message;
import javax.jms.MessageConsumer;
import javax.jms.MessageListener;
import javax.jms.MessageProducer;
import javax.jms.Session;
import javax.jms.TextMessage;
import javax.jms.Topic;

import org.apache.activemq.ActiveMQConnectionFactory;
import org.apache.activemq.pool.PooledConnectionFactory;

/**
 * Use in conjunction with TopicPublisher to test the performance of ActiveMQ
 * Topics.
 */
public class Subscriber implements MessageListener {

    private Connection connection;
    private MessageProducer producer;
    private Session session;
    private int count;
    private int sleep = 300;
    private long start;
    private Topic topic;
    private String subject;
    public static ConnectionFactory factory;

    private String url = Util.INSTANCE.getProviderUrl();

    private boolean throwExceptions = false;
    private boolean pooled;

    public Subscriber(String p_destination) {
        this.subject = p_destination;

    }

    /**
     * @param p_throwExceptions the throwExceptions to set
     */
    public void setThrowExceptions(boolean p_throwExceptions) {
        throwExceptions = p_throwExceptions;
    }

    public void setPooled(boolean pooled) {
        this.pooled = pooled;
    }

    public void setSleep(int sleep) {
        this.sleep = sleep;
    }

    public void run() throws JMSException {
        System.out.println("Config: throwExceptions="+this.throwExceptions);
        if (factory == null) {
            if(pooled) {
                PooledConnectionFactory f = new PooledConnectionFactory(url);
                f.setMaximumActive(1000);
                factory = f;
            } else {
                ActiveMQConnectionFactory f = new ActiveMQConnectionFactory(url);
                factory = f;
            }
        }

        System.out.println(this.subject+" - Create connection...");

        this.connection = factory.createConnection();
        this.connection.setExceptionListener(new OnExceptionListener(subject));

        System.out.println(this.subject+" - Create session...");

        session = connection.createSession(false, Session.AUTO_ACKNOWLEDGE);

        System.out.println(this.subject+" - Create topic...");

        topic = session.createTopic(this.subject);


        MessageConsumer consumer = session.createConsumer(topic);

        System.out.println(this.subject+" - Create consumer...");

        consumer.setMessageListener(this);

        System.out.println(this.subject+" - Connection start...");

        connection.start();


        System.out.println(this.subject+" - Waiting for messages...");
    }

    private static boolean checkText(Message m, String s) {
        try {
            return m instanceof TextMessage && ((TextMessage)m).getText().equals(s);
        } catch (JMSException e) {
            e.printStackTrace(System.out);
            return false;
        }
    }

    public void onMessage(Message message) {
        String excText;
        if (checkText(message, "SHUTDOWN")) {
            System.out.println("SHUTDOWN");
            try {
                //connection.close();
            } catch (Exception e) {
                e.printStackTrace(System.out);
            }
            excText = "SHUTDOWN";


        } else {

            if (count == 0) {
                start = System.currentTimeMillis();
            }

            if (++count % 1 == 0) {
                //System.out.println("Received " + count + " messages.");
                System.out.println("M-"+count);
            }

            excText = "M-"+count;

            if(sleep > 0) {
                try {
                    Thread.sleep(sleep);
                } catch (InterruptedException e) {
                    e.printStackTrace();
                }
            }
        }

        if(this.throwExceptions) {
            if(count % 2 == 0) {
                String ext = "Test exception: "+excText;
                System.out.println(ext);
                throw new RuntimeException(ext);
            }
        }
    }

    public void setUrl(String url) {
        this.url = url;
    }

}
